unit uToyShop;

interface

uses
  classes                   // for TStrings & TStringList
  , uToy                    // for TToy
  , SingletonInheritMany
  ;

type

  TToyShop = class(TSingleton)
  private

    FFactoryList: TStringList;

  protected

    procedure Initialise; override;
    procedure Cleanup; override;

  public

    function GetSet1(ToyType: string): TToy;
    function GetSet2(ToyType: string): TToy;
    function GetSet3(ToyType: string): TToy;
    procedure GetToyTypes(ToyTypes: TStrings);
    procedure Register(ToyType, ToyFactoryClass: string);

  published

  end;


implementation

uses
  sysutils                        // for Exception
  , uToyFactory
  , uBricksFactory
  , uDollsFactory
  ;


{ TToyShop }

procedure TToyShop.Cleanup;
var
  nFactory: integer;
begin
  for nFactory := 0 to Pred(FFactoryList.Count) do
    TToyFactory(FFactoryList.Objects[nFactory]).Free;
  FFactoryList.Free;
  inherited;
end;

function TToyShop.GetSet1(ToyType: string): TToy;
var
  nFactory: integer;
begin
  nFactory := FFactoryList.IndexOf(ToyType);
  if (nFactory > -1) then
    Result := TToyFactory(FFactoryList.Objects[nFactory]).GetSet1
  else
    raise Exception.Create(ToyType + ' is not a valid toy type');
end;

function TToyShop.GetSet2(ToyType: string): TToy;
var
  nFactory: integer;
begin
  nFactory := FFactoryList.IndexOf(ToyType);
  if (nFactory > -1) then
    Result := TToyFactory(FFactoryList.Objects[nFactory]).GetSet2
  else
    raise Exception.Create(ToyType + ' is not a valid toy type');
end;

function TToyShop.GetSet3(ToyType: string): TToy;
var
  nFactory: integer;
begin
  nFactory := FFactoryList.IndexOf(ToyType);
  if (nFactory > -1) then
    Result := TToyFactory(FFactoryList.Objects[nFactory]).GetSet3
  else
    raise Exception.Create(ToyType + ' is not a valid toy type');
end;

procedure TToyShop.GetToyTypes(ToyTypes: TStrings);
begin
  // Copy the list of toy types to the supplied string list
  ToyTypes.AddStrings(FFactoryList);
end;

procedure TToyShop.Initialise;
begin
  FFactoryList := TStringList.Create;
end;

procedure TToyShop.Register(ToyType, ToyFactoryClass: string);
var
  pcTemp: TPersistentClass;
begin
  pcTemp := FindClass(ToyFactoryClass);
  FFactoryList.AddObject(ToyType, pcTemp.Create);
end;

end.
